# https://www.datacamp.com/tutorial/tutorial-postgresql-python
# docker compose down && docker compose up -d && datacamp-postgres-tutorial
import psycopg2
import dotenv
import os 
# from icecream import ic


def main():
    dotenv.load_dotenv()
    print("Connecting to the PostgreSQL database...")
    conn = connect()
    cursor = conn.cursor()
    print("Creating a table")
    # SERIAL: auto-incrementing integer
    # VARCHAR: variable-length character string
    # (50) Set the maximum length of the course_name column to 50 characters.
    # UNIQUE: ensure that all values in the course_name column are unique.
    # NOT NULL: ensure that the course_name column cannot have NULL values.
    # (100) Set the maximum length of the course_instructor column to 100 characters.
    # (20) Set the maximum length of the topic column to 20 characters.
    cursor.execute("""CREATE TABLE datacamp_courses(
            course_id SERIAL PRIMARY KEY,
            course_name VARCHAR (50) UNIQUE NOT NULL,
            course_instructor VARCHAR (100) NOT NULL,
            topic VARCHAR (20) NOT NULL);
            """)
    print("Creating some records...")
    cursor.execute("INSERT INTO datacamp_courses(course_name, course_instructor, topic) VALUES('Introduction to SQL','Izzy Weber','Julia')");
    cursor.execute("INSERT INTO datacamp_courses(course_name, course_instructor, topic) VALUES('Analyzing Survey Data in Python','EbunOluwa Andrew','Python')");
    cursor.execute("INSERT INTO datacamp_courses(course_name, course_instructor, topic) VALUES('Introduction to ChatGPT','James Chapman','Theory')");
    cursor.execute("INSERT INTO datacamp_courses(course_name, course_instructor, topic) VALUES('Introduction to Statistics in R','Maggie Matsui','R')");
    cursor.execute("INSERT INTO datacamp_courses(course_name, course_instructor, topic) VALUES('Hypothesis Testing in Python','James Chapman','Python')");
    print("Committing the table to the database...")
    conn.commit()
    print("Fetching all the available rows...")
    cursor.execute('SELECT * FROM datacamp_courses;')
    rows = cursor.fetchall()
    for row in rows:
        print(row)
    print("Closing the connection and the cursor...")
    print("Changing topic \"Julia\" to \"SQL\"...")
    cursor.execute("UPDATE datacamp_courses SET topic = 'SQL' WHERE course_name = 'Introduction to SQL';")
    conn.commit()
    print("Deleting \"Introduction to Statistics in R\"...")
    cursor.execute("DELETE from datacamp_courses WHERE course_name = 'Introduction to Statistics in R';")
    conn.commit()
    print("Sorting table by instructor...")
    cursor.execute('SELECT * FROM datacamp_courses ORDER BY course_instructor')
    rows = cursor.fetchall()
    for row in rows:
        print(row)
    print("Fetching the number of courses each instructor teaches...")
    cursor.execute('SELECT course_instructor, COUNT(*) FROM datacamp_courses GROUP BY course_instructor')
    for row in cursor.fetchall():
        print(row)
    print("Creating the new table for language rankings...")
    # https://www.w3schools.com/sql/sql_primarykey.ASP
    cursor.execute("""
    CREATE TABLE programming_languages (
        language_id INT PRIMARY KEY,
        language_name TEXT NOT NULL,
        course_number INT NOT NULL,
        tiobe_ranking INT NOT NULL
    )
""")
    cursor.execute("""
    INSERT INTO programming_languages (language_id, language_name, course_number, tiobe_ranking) 
    VALUES 
    (1, 'SQL', 31, 8),
    (2, 'Python', 157, 1),
    (3, 'R', 132, 16),
    (4, 'Julia', 2, 33),
    (5, 'Scala', 1, 38)
""")
    conn.commit()
    # datacamp: "We will use an INNER JOIN to get only the information of the programming languages that appear in the datacamp_course table. "
    print("Getting courses, their instructors, the topic, and the tiobe ranking of the language...")
    cursor.execute("""SELECT course_name, course_instructor, topic, tiobe_ranking
FROM datacamp_courses
INNER JOIN programming_languages
ON datacamp_courses.topic = programming_languages.language_name""")
    for row in cursor.fetchall():
        print(row)
    cursor.close()
    conn.close()
    print("Now using SQLAlchemy to interact with the database...")
    import sqlalchemy
    engine = sqlalchemy.create_engine(f"postgresql://{os.getenv('POSTGRES_USER')}:{os.getenv('POSTGRES_PASSWORD')}@{os.getenv('DB_HOST')}:{os.getenv('DB_PORT')}/{os.getenv('POSTGRES_DB')}")
    conn = engine.connect()
    print("Fetching all the available rows...") 
    output = conn.execute(sqlalchemy.text("SELECT * FROM datacamp_courses"))
    print(output.fetchall())
    conn.close()


def connect():
    conn = psycopg2.connect(
        database=os.getenv('POSTGRES_DB'),
        host=os.getenv('DB_HOST'),
        port=os.getenv('DB_PORT'),
        user=os.getenv('POSTGRES_USER'),
        password=os.getenv('POSTGRES_PASSWORD')
    )
    return conn

if __name__ == "__main__":
    main()